import json

from aiohttp import web
from nacl.exceptions import BadSignatureError
from v6d0auth import certs

__all__ = ('get_app',)

from v6d1tokens.tdb import TDB


def define_routes(routes: web.RouteTableDef, tdb: TDB):
    print(certs.vkey.encode().hex())

    @routes.get('/')
    async def home(_request: web.Request):
        return web.Response(body='v6d0auth\n')

    @routes.post('/reg')
    async def reg(request: web.Request):
        try:
            await tdb.reg(await request.read())
        except BadSignatureError:
            raise web.HTTPUnauthorized
        except json.JSONDecodeError:
            raise web.HTTPBadRequest
        else:
            raise web.HTTPOk

    @routes.post('/get')
    async def get(request: web.Request):
        try:
            token_encrypted = await tdb.get(await request.read())
        except BadSignatureError:
            raise web.HTTPUnauthorized
        except json.JSONDecodeError:
            raise web.HTTPBadRequest
        except KeyError:
            raise web.HTTPNotFound
        else:
            return web.Response(body=token_encrypted)


def app_routes(tdb: TDB) -> web.RouteTableDef:
    routes = web.RouteTableDef()
    define_routes(routes, tdb)
    return routes


def app_with_routes(routes: web.RouteTableDef):
    app = web.Application()
    app.add_routes(routes)
    return app


def get_app(tdb: TDB) -> web.Application:
    return app_with_routes(app_routes(tdb))
